(function () {
  'use strict';

  /**
   * @typedef {Object} AccountAndWorkspace
   * @property {string} workspace - Current IO workspace
   * @property {string} account - Current IO account
   */

  /**
   * Gets the account and workspace in which this
   * script is running on top of.
   *
   * @returns {AccountAndWorkspace} The account and workspace in which this script is running on top of.
   */
  function getAccountAndWorkspace() {
    const [workspace, account] = window.location.host
      .replace(".vtexcommercestable.com.br", "")
      .replace(".vtexcommercebeta.com.br", "")
      .split("--");

    if (!account) {
      return { workspace: "master", account: workspace };
    }

    return { workspace, account };
  }

  /**
   * Whether or not the script is running inside an
   * iframe.
   *
   * @returns boolean
   */
  const isWithinIframe = (function isIframe() {
    return window.self !== window.top;
  })();

  const inContextScript = `var _jipt = [];
_jipt.push(['project', 'admin-home']);`;

  const CRWDIN_IN_CONTEXT_ID = "crowdin-in-context-legacy-id--DoUbNU";
  const CRWDIN_IN_CONTEXT_SRC_ID = "crowdin-in-context-src-legacy-id--DoUbNU";
  const CRWDIN_IN_CONTEXT_WORKSPACE = "acholi";
  const CRWDIN_IN_CONTEXT_ACCOUNT = "storecomponents";

  /**
   * Prepares the Crowdin In Context
   * initialization.
   *
   * @returns void
   */
  function prepareCrowdinInContext() {
    if (!document.getElementById(CRWDIN_IN_CONTEXT_ID)) {
      const script = document.createElement("script");
      script.id = CRWDIN_IN_CONTEXT_ID;
      script.type = "text/javascript";
      script.innerText = inContextScript;
      document.body.appendChild(script);
    }
  }

  /**
   * Loads the Crowdin In Context
   * script.
   *
   * @returns void
   */
  function loadCrowdinInContext() {
    if (!document.getElementById(CRWDIN_IN_CONTEXT_SRC_ID)) {
      const script = document.createElement("script");
      script.id = CRWDIN_IN_CONTEXT_SRC_ID;
      script.type = "text/javascript";
      script.src = "//cdn.crowdin.com/jipt/jipt.js";
      document.body.appendChild(script);
    }
  }

  /**
   * Checks whether or not the script
   * is running on the localization
   * workspace.
   *
   * @returns boolean
   */
  function shouldInjectCrowdinInContext() {
    if (!isWithinIframe) {
      return false;
    }

    const { account, workspace } = getAccountAndWorkspace();

    if (
      workspace === CRWDIN_IN_CONTEXT_WORKSPACE &&
      account === CRWDIN_IN_CONTEXT_ACCOUNT
    ) {
      return true;
    }

    return false;
  }

  /**
   * Injects the Crowdin In-Context script
   * on the legacy applications within the
   * Admin.
   *
   * @returns void
   */
  function startCrowdinInContext() {
    if (shouldInjectCrowdinInContext()) {
      prepareCrowdinInContext();
      loadCrowdinInContext();
    }
  }

  // TO UPDATE i18n version, deploy version and change tag here
  const I18N_VERSION = "v0.1.231";
  const FALLBACK_LANG = "en-US";

  function i18nextJqueryInit() {
    i18nextJquery.init(i18next, $, {
      tName: "t",
      i18nName: "i18n",
      handleName: "localize",
      selectorAttr: "data-i18n",
      targetAttr: "data-i18n-target",
      optionsAttr: "data-i18n-options",
      useOptionsAttr: false,
      parseDefaultValueFromContent: true,
    });
  }

  function handleLocaleSelected(_, lang = FALLBACK_LANG) {
    console.debug(
      `%c [ADMIN CATALOG JS] \n Fetching messages from i18n repo version: ${I18N_VERSION} for selected locale: `,
      "background: #002833; color: #258bd2",
      lang
    );
    $.get(`//io.vtex.com.br/i18n/${I18N_VERSION}/catalog/` + lang + ".json")
      .done(handleI18nData(lang))
      .fail(function () {
        console.debug(
          `%c [ADMIN CATALOG JS] \n Error fetching lang locale: ${lang}`,
          "background: #002833; color: #F71963"
        );
        handleI18nData(null); // this triggers the load again, but with fallback lang
      });
  }

  function handleDatesFormat(lang) {
    console.debug(
      `%c [ADMIN CATALOG JS] \n Handling dates format in lang: ${lang}`,
      "background: #002833; color: #258bd2"
    );
    const dateFormatOptions = {
      year: "numeric",
      month: "2-digit",
      day: "2-digit",
    };
    const dateTimeFormatOptions = {
      ...dateFormatOptions,
      hour: "2-digit",
      minute: "2-digit",
      second: "2-digit",
    };
    // DATE ELEMENTS
    var dateElements = $(".vtex-locale-date");
    // loop trough date elements
    $.each(dateElements, (_, e) => {
      // translate element text based on lang using iso date attribute
      const isoDate = $(e).attr("data-vtex-date-utc");
      const finalDate = new Date(isoDate).toLocaleDateString(
        lang,
        dateFormatOptions
      );
      if (finalDate && finalDate !== "Invalid Date") {
        $(e).text(finalDate);
      }
    });
    // DATE TIME ELEMENTS
    var dateTimeElements = $(".vtex-locale-datetime");
    // loop trough date time elements
    $.each(dateTimeElements, (_, e) => {
      // translate element text based on lang using iso date attribute
      const isoDate = $(e).attr("data-vtex-datetime-utc");
      const finalDate = new Date(isoDate).toLocaleDateString(
        lang,
        dateTimeFormatOptions
      );
      if (finalDate && finalDate !== "Invalid Date") {
        $(e).text(finalDate);
      }
    });
  }

  function handleI18nData(lang) {
    if (!lang) {
      // Load again in english if initial lang is not found
      console.debug(
        `%c [ADMIN CATALOG JS] \n Trying the fallback locale lang: ${FALLBACK_LANG}`,
        "background: #002833; color: #258bd2"
      );
      handleLocaleSelected(null, FALLBACK_LANG);
      return;
    }
    return function (messagesJson) {
      var resources = {};
      resources[lang] = { translation: messagesJson };
      i18next.init(
        {
          lng: lang,
          debug: true,
          resources: resources,
          fallbackLng: FALLBACK_LANG,
        },
        function () {
          $("body").localize();
        }
      );
      handleDatesFormat(lang);
    };
  }

  function addVtexLoaderScript() {
    const vtexLoaderScript = document.createElement("script");
    vtexLoaderScript.async = false;
    vtexLoaderScript.src =
      "//io.vtex.com.br/io-vtex-loader/2.3.1/io-vtex-loader.min.js";
    document.body.appendChild(vtexLoaderScript);
  }

  function fixBreadcrumbLinksToUseHTTPS() {
    $('#breadCrumbNav a').each(function() {
      const breadcrumbLink = $(this);
      const breadcrumbUrl = breadcrumbLink.attr("href") || '';

      if (!breadcrumbUrl.startsWith("https://")) {
        breadcrumbLink.attr('href', breadcrumbUrl.replace('http://', 'https://'));
      }
    });
  }

  function tryToRedirectUserFromLegacyPageToAdmin() {
    const pathMaps = JSON.parse(window.localStorage.getItem('__paths_to_redirect') || '{}');

    Object.entries(pathMaps).forEach(function ([legacyPage, adminPage]) {
      const pathName = window.location.pathname.toLowerCase();
      if (adminPage && pathName.endsWith(legacyPage.toLowerCase())) {
        window.parent.postMessage(
          {
            type: "admin.innerNavigation",
            destination: adminPage,
          },
          "*"
        );
      }
    });
  }

  function handleTopbarLoaded() {
    vtex.topbar.topbar = new vtex.topbar.Topbar();
  }

  function handleIframePostMessage(e) {
    if (!(e.data && e.data.action)) return;
    
    if (e.data.action.type === "LOCALE_SELECTED") {
      handleLocaleSelected(e, e.data.action.payload);
    }

    if (e.data.action.type === "PATHS_TO_REDIRECT") {
      window.localStorage.setItem('__paths_to_redirect', e.data.action.payload);
    }
  }

  function handleUnload(_) {
    const focusedElement = $(":focus")[0];
    const destination = focusedElement ? focusedElement.href : null;
    if (
      destination &&
      destination.includes("/admin") &&
      !(
        destination.includes("/admin/Site") ||
        destination.includes("/admin/Control")
      )
    ) {
      console.debug(
        `%c [ADMIN CATALOG JS] \n Will navigate to other domain: ${destination}`,
        "background: #002833; color: #258bd2"
      );
      window.top.postMessage(
        {
          type: "admin.absoluteNavigation",
          destination,
        },
        "*"
      );
    }
  }

  function setGlobals(objects) {
    objects.forEach((object) => {
      if (typeof object === "function") {
        window[object.name] = object;
      } else {
        window[Object.keys(object)[0]] = Object.values(object)[0];
      }
    });
  }

  console.debug(
    `%c [ADMIN CATALOG JS] \n Running custom js `,
    "background: #002833; color: #258bd2"
  );

  // If the current page should redirect the user to an admin page,
  // we trigger an admin.ioAppNavigation that should be handled by the iframe that's rendering this page.
  tryToRedirectUserFromLegacyPageToAdmin();

  $(function ($) {
    /*****************************
     *   THIS BLOCK ALWAYS RUNS   *
     ******************************/

    // Initialize jQuery i18n
    i18nextJqueryInit();

    var pathname = window.location.pathname;

    if (pathname.toLocaleLowerCase().includes("site/categories.aspx")) {
      // open categories automatically if category page
      var colapseBtns = $(".hitarea.expandable-hitarea.lastExpandable-hitarea");
      if (colapseBtns.length > 0) {
        colapseBtns[0].click();
      } else {
        setTimeout(() => {
          $(".hitarea.expandable-hitarea.lastExpandable-hitarea")[0].click();
        }, 420); // blaze it
      }
      // highlight of searched category style fix (desculpe mundo)
      var stylesheet = document.styleSheets[document.styleSheets.length - 1];
      for (var i in document.styleSheets) {
        if (
          document.styleSheets[i].href &&
          document.styleSheets[i].href.indexOf("everything-else.css")
        ) {
          stylesheet = document.styleSheets[i];
          break;
        }
      }
      if (stylesheet.addRule) {
        stylesheet.addRule(
          ".label-success",
          "color:#1346d8 !important;background-color: #f2f4f5 !important"
        );
      } else if (stylesheet.insertRule) {
        stylesheet.insertRule(
          ".label-success { color:#1346d8 !important;background-color: #f2f4f5 !important }",
          stylesheet.cssRules.length
        );
      }
    }

    /*************************************************
     *   THIS BLOCK RUNS ONLY WHEN INSIDE AN IFRAME   *
     **************************************************/

    if (isWithinIframe) {
      console.debug(
        `%c [ADMIN CATALOG JS] \n running inside iframe`,
        "background: #002833; color: #258bd2"
      );
      window.addEventListener("message", handleIframePostMessage);
      // Monitor navigation before it happens, if its an admin (not catalog) navigate to right url
      window.addEventListener("beforeunload", handleUnload);
      // Let the parent frame know details about our navigation AFTER navigating
      window.top.postMessage(
        {
          type: "admin.navigation",
          hash: window.location.hash,
          search: window.location.search,
          pathname: window.location.pathname,
        },
        "*"
      );
      // hides menu and header titles when inside iframe since there is already a header there
      $(".AspNet-Menu").parent().parent().hide();
      $(".barra-alerta").hide(); // esconde barra-alerta
      // $('#breadCrumbNav').hide()
      // Fixes some breadcrumbs URL that aren't using HTTPS
      fixBreadcrumbLinksToUseHTTPS();
      if (!window.location.href.includes("Site/RelatorioIndexacao.aspx")) {
        $("#content h2").hide();
      } else {
        // in RelatorioIndexacao.aspx page, header is a div with an h3
        $(".page-header").hide();
        // ajusta largura para ficar grid com 3 cards ao inves de 2 com menu lateral aberto
        $(".container")[0].style.width = "100%";
      }
      $("#areaUsuario").hide();
      // dispatch event to myvtex sending content height to update iframe size
      var currentHeight = document.body.scrollHeight;
      setInterval(() => {
        var newHeight = document.body.scrollHeight;
        if (currentHeight !== newHeight) {
          currentHeight = newHeight;
          window.parent.postMessage(
            {
              type: "admin.updateContentHeight",
              height: newHeight,
            },
            "*"
          );
        }
      }, 1000);

      startCrowdinInContext();
    }

    /***************************************************
     *  THIS BLOCK RUNS ONLY WHEN NOT INSIDE AN IFRAME  *
     ****************************************************/

    if (window.self === window.top) {
      console.debug(
        `%c [ADMIN CATALOG JS] \n NOT running inside iframe`,
        "background: #002833; color: #258bd2"
      );
      // legacy topbar load
      addVtexLoaderScript();
      $(window).on("localeSelected.vtex", handleLocaleSelected);
      $(window).on("topbarLoaded.vtex", handleTopbarLoaded);
      // style changes
      $("#areaUsuario").children().removeClass("bt");
      $("#ctl00_AreaUsuario_AreaUsuario1_lnkAcesseSite").removeClass("bt");
      $("#ctl00_AreaUsuario_AreaUsuario1_lnkAcesseSiteAdminBeta").removeClass(
        "bt"
      );
      $("#areaUsuario")[0].style["margin-top"] = "20px";
      // start marking correct menu item as active
      var aspNetMenuChildren = $(".AspNet-Menu").children();
      var checkIfPathIsActive = (href) => {
        return href && href.includes(pathname);
      };
      var titlePathDictionary = {
        "mainMenu.products.label": [
          "Site/Produto.aspx",
          "Site/Categories.aspx",
          "Site/Marca.aspx",
          "Site/Relatorio_Skus.aspx",
          "Site/ProdutoExportacaoImportacaoEspecificacaoV2.aspx",
          "Site/ProdutoExportacaoImportacaoEspecificacaoSKUV2.aspx",
          "Site/SkuTabelaValor.aspx",
          "Site/gerarimagens.aspx",
          "Site/ProdutoImagemExportacao.aspx",
          "Site/ProdutoExportacaoImportacaoAvaliacao.aspx",
          "Site/Anexo.aspx",
          "Site/SkuServicoTipo.aspx",
          "Site/SkuServicoValor.aspx",
          "Site/SkuVincularValorServico.aspx",
          "Site/Relatorio_Seguranca.aspx",
          "Site/LinksRelatorios.aspx",
          "Site/GiftList.aspx",
          "Site/RelatorioIndexacao.aspx",
          "Site/Relatorio_AviseMeSku.aspx",
          "Site/RelatorioNews.aspx",
          "Site/Resenha.aspx",
          "Site/SkuCondicaoComercial.aspx",
          "Site/Fornecedor.aspx",
        ],
        "mainMenu.campaigns.vouchers": ["Site/Vale.aspx"],
        "mainMenu.settings.label": [
          "Site/ConfigForm.aspx",
          "Site/ConfigSEOContents.aspx",
          "Site/TextoSite.aspx",
          "Site/GiftListType.aspx",
          "Site/Xml.aspx",
          "Site/TipoArquivo.aspx",
          "Site/GeographicRegion.aspx",
        ],
        "mainMenu.marketplace.label": [
          "Site/Store.aspx",
          "Site/Seller.aspx",
          "Site/SkuSeller.aspx",
        ],
      };
      var forceTitleActiveStyle = (el) => {
        $.each(titlePathDictionary[el.children[0].dataset.i18n], (_, path) => {
          if (pathname.includes(path)) {
            if (el.style) {
              el.children[0].style["border-bottom"] = "1px solid #368df7";
            } else {
              el.children[0].style = {};
              el.children[0].style["border-bottom"] = "1px solid #368df7";
            }
          }
        });
      };
      var forceSubItemActiveStyle = (el) => {
        if (el.style) {
          el.style["background-color"] = "#ddd";
        } else {
          el.style = {};
          el.style["background-color"] = "#ddd";
        }
      };

      // loop trough menu items
      $.each(aspNetMenuChildren, (_, e) => {
        // loop trough menu item children (they are all LI tags)
        $.each(e.children, (__, _e) => {
          if (_e.tagName === "A") {
            forceTitleActiveStyle(_e);
          } else {
            // if it's not tag A it's UL
            // loop trough UL children
            $.each(_e.children, (___, __e) => {
              // here every __e is an LI, so loop again
              $.each(__e.children, (____, ___e) => {
                if (___e.tagName === "A") {
                  if (checkIfPathIsActive(___e.href)) {
                    forceSubItemActiveStyle(___e);
                  }
                } else {
                  // if it's not tag A it's UL again
                  // loop trough UL children again
                  $.each(___e.children, (_____, ____e) => {
                    if (checkIfPathIsActive(____e.children[0].href)) {
                      forceSubItemActiveStyle(____e.children[0]);
                    }
                  });
                }
              });
            });
          }
        });
      });
      // end marking correct menu item as active
    }
  });

  setGlobals([
    handleLocaleSelected,
    handleIframePostMessage,
    handleTopbarLoaded,
    handleUnload,
    i18nextJqueryInit,
    addVtexLoaderScript,
    handleDatesFormat,
    handleI18nData,
    { I18N_VERSION },
    { FALLBACK_LANG },
  ]);

}());
